//-----------------------------------------------------------------------
//	File:		$RCSfile: VideoEncoder.v,v $
//	Version:	$Revision: 1.1 $
//	Desc:		ITU656/601 Video Encoder
//	Author:		Greg Gibeling
//	Copyright:	Copyright 2003 UC Berkeley
//	This copyright header must appear in all derivative works.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Change Log
//-----------------------------------------------------------------------
//	$Log: VideoEncoder.v,v $
//	Revision 1.1  2004/10/05 19:56:01  SYSTEM
//	Initial Import
//	
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Includes
//-----------------------------------------------------------------------
`include "Const.v"
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:		VideoEncoder
//	Desc:		This module will request pixel pairs from a host
//			once every four clock cycles during the active
//			video period.
//
//			The interface allows for synchronous read in that
//			it will expect data on the cycle after it makes
//			the request.
//
//			Pixels will be requested across each line and
//			then down across first the odd frame then
//			similarly across the even frame.  Note that line
//			adresses will be every-other-one due to this
//			interlacing.
//	Params:		totallines:	The total number of video lines
//					to output.
//			activelines:	The total number of active video
//					lines to output
//			f0topblank:	The number of vertical blanking
//					lines to insert above the active
//					portion of the 0 (Odd) field.
//			hblanksamples:	The number of samples (16bit
//					words) per line which are
//					blanking.  Includes SAV/EAV
//			activesamples:	The number of active video samples
//					per line.
//-----------------------------------------------------------------------
module	VideoEncoder(	//-----------------------------------------------
			//	Video Encoder I/O
			//-----------------------------------------------
			VE_P,		// Video data to the ADV7194
			VE_SCLK,	// I2C Clock
			VE_SDA,		// I2C Data
			VE_PAL_NTSC,	// PAL/NTSC Video Standard Select
			VE_RESET_B_,	// Reset out to ADV7194
			VE_HSYNC_B_,	// Unused
			VE_VSYNC_B_,	// Unused
			VE_BLANK_B_,	// Unused
			VE_SCRESET,	// Unused
			VE_CLKIN,	// Clock out to ADV7194
			//-----------------------------------------------

			//-----------------------------------------------
			//	System Inputs
			//-----------------------------------------------
			Clock,
			Reset,
			//-----------------------------------------------

			//-----------------------------------------------
			//	Video Data Host Interface
			//-----------------------------------------------
			DIn,
			InRequest,
			InRequestLine,
			InRequestPair
			//-----------------------------------------------
		);
	//---------------------------------------------------------------
	//	Parameters
	//		These could theoretically change if we changed
	//		the video format.  We don't expect you to plan
	//		for that.  You may assume these are CONSTANTS!
	//---------------------------------------------------------------
	parameter		totallines =		525,	// Total number of video lines
				activelines =		507,	// Number of active video lines
				f0topblank =		6,	// Number of blank lines at the top of Field0
				hblanksamples =		138,	// Number of blank samples per line
				activesamples =		720;	// Number of active samples per line
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Constants
	//		Since the above constants are a little tricky to
	//		actually use, these values are provided to make
	//		your life easier.  You will probably want to
	//		do comparisons against these values.
	//---------------------------------------------------------------
	`ifdef MACROSAFE
	localparam		oddlines =		totallines >> 1,			// Total lines in odd field
				evenlines =		totallines - oddlines,			// Total lines in even field
				evenactive =		activelines >> 1,			// Active lines in even field
				oddactive =		activelines - evenactive,		// Active lines in odd field
				oddtopblank =		f0topblank,				// Blank lines at the top of the odd field
				eventopblank =		f0topblank + 1,				// Blank lines at the top of even field
				oddbottomblank =	oddlines - oddactive - oddtopblank,	// Blank lines at the bottom of odd field
				evenbottomblank =	evenlines - evenactive - eventopblank,	// Blank lines at the bottom of even field
				hblankbytes =		(hblanksamples * 2) - 8,		// Blank bytes per line (Notice subtract for EAV/SAV)
				activebytes =		(activesamples * 2),			// Active bytes per line;
				vcbits =		`log2(oddactive),			// Vertical Counter Bits
				hcbits =		`log2(activebytes);			// Horizontal Counter Bits
	`endif
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Debugging Prinouts
	//		These are for your convenience, only.  You might
	//		want to check that the right values are printed
	//		when you simulate this module.
	//---------------------------------------------------------------
	`ifdef MODELSIM
	initial begin
		$display("%m : VideoEncoder");
		$display("    totallines =      %d", totallines);
		$display("    activelines =     %d", activelines);
		$display("    f0topblank =      %d", f0topblank);
		$display("    hblanksamples =   %d", hblanksamples);
		$display("    activesamples =   %d", activesamples);
		$display("    oddlines =        %d", oddlines);
		$display("    evenlines =       %d", evenlines);
		$display("    evenactive =      %d", evenactive);
		$display("    oddactive =       %d", oddactive);
		$display("    oddtopblank =     %d", oddtopblank);
		$display("    eventopblank =    %d", eventopblank);
		$display("    oddbottomblank =  %d", oddbottomblank);
		$display("    evenbottomblank = %d", evenbottomblank);
		$display("    hblankbytes =     %d", hblankbytes);
		$display("    activebytes =     %d", activebytes);
		$display("    vcbits =          %d", vcbits);
		$display("    hcbits =          %d", hcbits);
	end
	`endif
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Video Encoder I/O
	//		I/O connections to the ADV7194 Video Encoder chip
	//---------------------------------------------------------------
	output	[9:0]		VE_P;
	inout			VE_SCLK, VE_SDA;
	output			VE_PAL_NTSC;
	output			VE_RESET_B_;
	output			VE_HSYNC_B_, VE_VSYNC_B_, VE_BLANK_B_;
	output			VE_SCRESET;
	output			VE_CLKIN;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	System Inputs
	//		System level clock and reset.  Always 27MHz.
	//---------------------------------------------------------------
	input			Clock, Reset;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Video Data Host Interface
	//		Interface to support access to SRAM.  Can be
	//		adapated to connect to SDRAM or anything else.
	//---------------------------------------------------------------
	input	[31:0]		DIn;
	output			InRequest;
	// Notice this doesn't have a -1, that lowest bit should the field, the [vcbits:1] should be the line within it
	output	[vcbits:0]	InRequestLine;
	output	[hcbits-3:0]	InRequestPair;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	I2C Operation Encodings (DO NOT CHANGE)
	//		Don't worry too much about this section, and
	//		be sure not to change it.
	//---------------------------------------------------------------
	localparam		OP_NOP =		2'b00,
				OP_Read =		2'b01,
				OP_Write =		2'b10,
				OP_Restart =		2'b11;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Wires and Regs
	//---------------------------------------------------------------
	wire	[2:0]		I2COpCount;				// DO NOT CHANGE
	reg	[1:0]		I2COpCode;				// DO NOT CHANGE
	reg	[7:0]		I2CTXData;				// DO NOT CHANGE
	wire			I2CRequest, I2CDone;			// DO NOT CHANGE

	//	Add wire and reg declarations here

	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Assigns and Decodes
	//		Here are a set of assigns which are all very
	//		simple.  Most of them are setting the constant
	//		inputs to the ADV7194 video encoder
	//---------------------------------------------------------------
	assign	VE_PAL_NTSC =			1'b0;			// DO NOT CHANGE
	assign	VE_RESET_B_ =			~Reset;			// DO NOT CHANGE
	assign	VE_HSYNC_B_ =			1'b1;			// DO NOT CHANGE
	assign	VE_VSYNC_B_ =			1'b1;			// DO NOT CHANGE
	assign	VE_BLANK_B_ =			1'b1;			// DO NOT CHANGE
	assign	VE_SCRESET =			1'b0;			// DO NOT CHANGE
	assign	VE_CLKIN =			Clock;			// DO NOT CHANGE

	// You will need to use I2CDone to know when the ADV7194 has been
	//	fully configured.  You shouldn't try to send anything out
	//	to the ADV7194 until this signal goes high.
	assign	I2CDone =			I2COpCount[2];		// DO NOT CHANGE
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	I2C Operation Counter (DO NOT CHANGE)
	//---------------------------------------------------------------
	Counter #(3)	I2CCnt(	.Clock(		Clock),
				.Reset(		Reset),
				.Set(		1'b0),
				.Load(		1'b0),
				.Enable(	I2CRequest & ~I2CDone),
				.In(		3'h0),
				.Count(		I2COpCount));
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	I2C Bus Master (DO NOT CHANGE)
	//---------------------------------------------------------------
	I2CMaster	I2C(	.SDA(		VE_SDA),
				.SCL(		VE_SCLK),
				.InOpCode(	I2COpCode),
				.DIn(		I2CTXData),
				.DOut(		),
				.InRequest(	I2CRequest),
				.InAck(		),
				.OutValid(	),
				.Clock(		Clock),
				.Reset(		Reset));
	defparam	I2C.clockfreq =		27000000;
	defparam	I2C.i2crate =		400000;
	defparam	I2C.OP_NOP =		OP_NOP;
	defparam	I2C.OP_Read =		OP_Read;
	defparam	I2C.OP_Write =		OP_Write;
	defparam	I2C.OP_Restart =	OP_Restart;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	I2C Operation Generator (DO NOT CHANGE)
	//---------------------------------------------------------------
	always @ ( * ) begin
		case (I2COpCount)
			3'h0: {I2COpCode, I2CTXData} =				{OP_NOP, 8'hxx};
			3'h1: {I2COpCode, I2CTXData} =				{OP_Write, 8'h54};
			3'h2: {I2COpCode, I2CTXData} =				{OP_Write, 8'h02};
			3'h3: {I2COpCode, I2CTXData} =				{OP_Write, 8'h60};
			default: {I2COpCode, I2CTXData} =			{OP_NOP, 8'hxx};
		endcase
	end
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Checkpoint2
	//---------------------------------------------------------------

	// We recommend that you add more verilog into this area...

	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------